display_application_commands, disac

        Displays a list of legal commands for this application.

          Examples:

            disac              # the `brief' display
            disac -do f        # the `full' display
.display_option
        Specifies the level of output desired.
.output
        Specifies the name of the file to write information to.
