sample_command, sc

	This is a simple command that demonstrates how evaluate_parameters
	can be embedded into your application.  

	  Examples:
 
	    sample_command -full_help
	    samc -id "any 'ol string" red blue green
.id
	The `id' parameter is just a string.	
