add_message_modules, add_message_module, addmm

	Adds message modules to an ar message module archive file.

	  Example:
	
	    addmm -mml my_mm_library my.mm

        This example adds the contents of the file my.mm to a personal
        message module archive file my_mm_library.

	If the archive library file is not specified it defaults to the
        installation message module library.

          Example:

            addmm message_module/*.mm

        This example adds all the message modules in the message_module
        catalog to the installation message module library.

	To examine a sample Message Module try this:

	  ar p /usr/local/lib/libevapmm.a sample.mm | more
.message_module_library
	Specifies the path name of the ar archive library to hold
	the text of the indicated message modules.

	Message modules contain informative text about the command
	they describe as well as parameter help data.
