#include <stdio.h>
#include "./evap.h"
#include "../pdt/sample_pdt_out"

int main(int argc,
	 char *argv[]);

int main (int argc, char *argv[])

/* evap 1 - array version, unconverted values, no types, no lists */

{
  int i;

  evap (&argc, &argv, pdt, NULL, pvt); /* evaluate parameters */
  printf("\nProgram name:\n  %s\n", pvt[P_HELP].unconverted_value);

  if (P_NUMBER_OF_PARAMETERS > 1 ) {
    printf("\nCommand line parameters and values:\n");
    for (i=1; i < P_NUMBER_OF_PARAMETERS; i++)
      printf("  Parameter %s=\"%s\"\n", pvt[i].parameter,
            pvt[i].unconverted_value);
  }

  if (argc > 1) {
    printf("\nFile names:\n  ");
    for(i=1; i < argc; i++)
      printf("%s%s", argv[i], (i < argc-1) ? " " : "");
    printf("\n");
  }

  exit( 0 );

} /* end main */
