sample_application, sama

	`sample_application' shows how to embed evaluate_parameters
	into your application.  Therefore, all the benefits of using
        evaluate_parameter on the command line apply to your
	application's command processing as well.

	Once inside the application, use `display_application_commands'
	to see a list of valid commands for this application.

	For each command, you can specify `-?', `-brief_help' or
	`-full_help' to see command and parameter help information.
.input
        Specifies the name of the input file.
