display_command_information, disci

	Displays information about a command.

	  Example:
	
	    disci -do f genpdt

        This example uses display_option full to see further information.
        Currently this additional data includes where the command was
        found and the name of the evaluate_parameters message module,
	text describing the command, as well as parameter help, if
	available.
.output
	Specifies the path name of the file to receive the Display
	Command Information output.
.display_option
	Specifies the level of help desired.  
.do_page
	Specifies whether `display_command_information' pages help
	output.  If paging is enabled the environment variable
	MANPAGER (or PAGER) is used to control the display of the
	generated help information.  If defined and non-null, the
	value of the environment variable is taken as the name of
	the program to pipe the help output through.  If no paging
	program is defined then the program `more' is used.

	The boolean environment variable D_DISCI_DO_PAGE can be
	set to FALSE/NO/OFF/0, any case, to disable this automatic
	paging feature (or you can set your paging program to
	`cat').
