
Program name:
  ./xxx

verbose = 1

command = "ps -el"
'list of' command2 = "s1", "s2", "s3", "s4"

scale_factor = 0.123409
'list of' scale_factor2 = "123.567", "-0.0045e-3"

millisecond_update_interval = 123
'list of' millisecond_update_interval2 = "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12"

ignore_output_file_column_one = 0
'list of' ignore_output_file_column_one2 = "1", "1", "1", "1", "0", "0", "0", "0"

output = >-
'list of' output2 = "/home/lusol/f1", "/home/lusol/f2", "/home/lusol/f3", "/home/lusol/f4"

queue = printer
'list of' queue2 = "printer", "plotter"

destination = Turkey.CC.Lehigh.EDU
'list of' destination2 = "Turkey.CC.Lehigh.EDU", "ap1", "ap2"

tty = /dev/tty99
'list of' tty2 = "/dev/console", "/dev/tty0", "/dev/tty1"

File names:
  z
