!

        Issue one or more commands to the shell.  If the SHELL environment
        variable is not defined or is empty, then /bin/sh is used.


          Examples:

            !date
            !del *.o; ls -al
