Command Source:  ./xxx



sample

	A sample program demonstrating typical evaluate_parameters
	usage.

	Examples:

	  sample
	  sample -usage_help
	  sample -?
	  sample -full_help
	  sample -mui 1234

Parameters:

-help, ?, usage_help, full_help: Display Command Information

	Display information about this command, which includes
	a command description with examples, plus a synopsis of
	the command line parameters.  If you specify -full_help
	rather than -help complete parameter help is displayed
	if it's available.

-verbose, v: switch

	A 'switch' type parameter emulates a typical standalone
	switch. If the switch is specified evaluate_parameters
	returns a '1'.

-command, c: string = D_SAMPLE_COMMAND, "ps -el"

	A 'string' type parameter is just a list of characters,
	which must be quoted if it contains whitespace. 
	NOTE:  for this parameter you can also create and
	initialize the environment variable D_SAMPLE_COMMAND to
	override the standard default value for this command
	line parameter.  All types except 'switch' may have a
	default environment variable for easy user customization.

-command2, c2: list of string = D_SAMPLE_COMMAND, ("s1", "s2", "s3", "s4")


-scale_factor, sf: real = 1.2340896e-1

	A 'real' type parameter must be a real number that may
	contain a leading sign, a decimal point and an exponent.

-scale_factor2, sf2: list of real = (123.567, -0.0045e-3)


-millisecond_update_interval, mui: integer = $required

	An 'integer' type parameter must consist of all digits
	with an optional leading sign.  NOTE: this parameter's
	default value is '$required', meaning that
	evaluate_parameters ensures that this parameter is
	specified and given a valid value.  All types except
	'switch' may have a default value of '$required'.

-millisecond_update_interval2, mui2: list of integer = (1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12)


-ignore_output_file_column_one, iofco: boolean = no

	A 'boolean' type parameter may be TRUE/YES/ON/1 or
	FALSE/NO/OFF/0, either upper or lower case.  If TRUE,
	evaluate_parameters returns a value of '1', else '0'.

-ignore_output_file_column_one2, iofco2: list of boolean = (TRUE, true, yes, 1, 0, no, FALSE, false)


-output, o: file = stdout

	A 'file' type parameter expects a filename.  For Unix
	$HOME and ~ are expanded.  For evap/Perl 'stdin' and
	'stdout' are converted to '-' and '>-' so they can be
	used in a Perl 'open' function.

-output2, o2: list of file = ($HOME/f1, ~/f2, $HOME/f3, ~/f4)


-queue, q: key plotter, postscript, text, printer, keyend = printer

	A 'key' type parameter enumerates valid values.  Only the
	specified keywords can be entered on the command line.

-queue2, q2: list of key plotter, postscript, text, printer, keyend = (printer, plotter)


-destination, d: application = `hostname`

	An 'application' type parameter is not type-checked in
	any way - the treatment of this type of parameter is
	application specific.  NOTE:  this parameter's default
	value is enclosed in grave accents (or "backticks").
	evaluate_parameters executes the command and uses it's
	standard output as the default value for the parameter.

-destination2, d2: list of application = (`hostname`, ap1, ap2)


-tty, t: name = /dev/tty99

	A 'name' type parameter is similar to a string except
	that embedded white-space is not allowed.  NOTE: this
	parameter is also a LIST, meaning that it can be
	specified multiple times and that each value is pushed
	onto a LIST of variable values.  In general you should
	quote all list elements.  All types except 'switch' may
	be 'list of'.

-tty2, t2: list of name = (/dev/console, /dev/tty0, /dev/tty1)


file(s) required by this command

