PDT (sample.mm) qa
  verbose, v: switch
  command, c: string = D_SAMPLE_COMMAND, "ps -el"
  command2, c2: list of string = D_SAMPLE_COMMAND, ("s1", "s2", "s3", "s4")
  scale_factor, sf: real = 1.2340896e-1
  scale_factor2, sf2: list of real = (123.567, -0.0045e-3)
  millisecond_update_interval, mui: integer = $required
  millisecond_update_interval2, mui2: list of integer = (1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12)
  ignore_output_file_column_one, iofco: boolean = no
  ignore_output_file_column_one2, iofco2: list of boolean = (TRUE, true, yes, 1, 0, no, FALSE, false)
  output, o: file = stdout
  output2, o2: list of file = ($HOME/f1, ~/f2, $HOME/f3, ~/f4)
  queue, q: key plotter, postscript, text, printer, keyend = printer
  queue2, q2: list of key plotter, postscript, text, printer, keyend = (printer, plotter)
  destination, d: application = `hostname`
  destination2, d2: list of application = (`hostname`, ap1, ap2)
  tty, t: name = /dev/tty99
  tty2, t2: list of name = (/dev/console, /dev/tty0, /dev/tty1)
PDTEND required_file_list
