#!/usr/local/bin/perl -w

require "src/evap.pl";

$file="SOL/QA/qa_pdt.pl";
open( PDT, "$file" );
@PDT = <PDT>;
close( PDT );

$file="SOL/QA/sample.mm";
open( PDT, "$file" );
@MM = <PDT>;
close( PDT );
@MM = grep( @_ = s/\n$//, @MM );

&evap( *PDT, *MM );

printf( "\nProgram name:\n  %s\n", $options{'help'} );

printf( "\nverbose = %d\n", $options{'verbose'} );

printf( "\ncommand = \"%s\"\n", $options{'command'} );
&print_list( 'command2' );

printf( "\nscale_factor = %f\n", $options{'scale_factor'} );
&print_list( 'scale_factor2' );

printf( "\nmillisecond_update_interval = %d\n", $options{'millisecond_update_interval'} );
&print_list( 'millisecond_update_interval2' );

printf( "\nignore_output_file_column_one = %d\n", $options{'ignore_output_file_column_one'} );
&print_list( 'ignore_output_file_column_one2' );

printf( "\noutput = %s\n", $options{'output'} );
&print_list( 'output2' );

printf( "\nqueue = %s\n", $options{'queue'} );
&print_list( 'queue2' );

printf( "\ndestination = %s\n", $options{'destination'} );
&print_list( 'destination2' );

printf( "\ntty = %s\n", $options{'tty'} );
&print_list( 'tty2' );

print "\nFile names:\n  ", join( ' ', @ARGV ), "\n" if @ARGV;




sub print_list {
    local( $p ) = @_;

    local (*glob) = 'opt_' . $p;
    print "'list of' $p = \"", join( '", "', @glob ), "\"\n";

}
